package reasoner.grounding;

import gnu.trove.set.hash.TIntHashSet;
import reasoner.ReasonerConfig;
import util.provider.AbstractProvider;

public class IntervalProvider extends AbstractProvider<Interval> {
  // Lazy Singleton
  private static IntervalProvider instance;
  
  private IntervalProvider() throws Exception {
    super(IntervalProvider.class.getName(), 1024);
  }
  
  public static IntervalProvider getInstance() throws Exception {
    if (instance == null) {
      instance = new IntervalProvider();
    }
    return instance;
  }

  @Override
  protected Interval getNew() {
    return new Interval();
  }
  
  public Interval get(int begin, int end, TIntHashSet lineageIds) throws Exception {
    return get(begin, end, lineageIds.toArray());
  }
  
  public Interval get(int begin, int end, int... lineageIds) throws Exception {
    Interval toReturn = getCleanObject();
    if (ReasonerConfig.debugMode) {
      if (begin >= end
      		&& (begin != Integer.MAX_VALUE && end != Integer.MIN_VALUE)) // used initially for writing to the database
      {
        throw new Exception("Invalid Interval: " + begin + " " + end);
      }
    }
    toReturn.begin = begin;
    toReturn.end = end;
    toReturn.lineageArrayIds.addAll(lineageIds);
    return toReturn;
  }
}