package reasoner.io;

import java.util.HashSet;

import reasoner.ReasonerConfig;
import reasoner.grounding.Tuple;
import reasoner.grounding.Interval;
import reasoner.lineage.Lineage;
import reasoner.lineage.LineageArray;
import reasoner.query.QueryExecuter.Results;
import util.io.FileWriter;

public class QueryStatisticsWriter {
	private FileWriter writer;
	private LineageArray array;
	
	public QueryStatisticsWriter(String path) throws Exception {
		writer = new FileWriter(path);
		writer.append("MaxShannonPerNode\tDebugMode\tConstant\tRun\t#Results\tSumLeafs\tSumMaxShannon\tSumTimeLimits\tConstraintClauses\tConstraintLeafs\tConstraintMaxShannons\tGroundingTime\tConstraintGroundingTime\tpreprocessingTime\tIntervalCompTime\tConfCompTime\n");
		array = LineageArray.getInstance();
	}
	
	public void appendResults(String constant, int run, Results results) throws Exception {
		writer.append(ReasonerConfig.maxShannonPerNode + "\t");
		writer.append(ReasonerConfig.debugMode + "\t");
		writer.append(constant + "\t");
		writer.append(run + "\t");
		writer.append(results.resultTuples.size() + "\t");
		int sumTimeLimits = 0;
		int sumLeafs = 0;
		int sumMaxShannon = 0;
		for (Tuple fact : results.resultTuples) {
			HashSet<Integer> timeLimits = new HashSet<Integer>();
			for (Interval interval : fact.intervals) {
				timeLimits.add(interval.begin);
				timeLimits.add(interval.end);
				Lineage lineage = array.getLineage(interval.lineageArrayIds.toArray()[0]);
				sumLeafs += lineage.vars.size();
				sumMaxShannon += lineage.getMaxShannon();
			}
			sumTimeLimits += timeLimits.size();
		}
		writer.append(sumLeafs + "\t");
		writer.append(sumMaxShannon + "\t");
		writer.append(sumTimeLimits + "\t");
		if (results.constraintLineage != null) {
			writer.append(results.numConstraintClauses + "\t");
			writer.append(results.constraintLineage.vars.size() + "\t");
			writer.append(results.constraintLineage.getMaxShannon() + "\t");
		} else {
			writer.append("-1\t");
			writer.append("-1\t");
			writer.append("-1\t");
		}
		writer.append(results.ruleGroundingTime + "\t");
		writer.append(results.constraintGroundingTime + "\t");
		writer.append(results.preprocessingTime + "\t");
		writer.append(results.intervalCompTime + "\t");
		writer.append(results.confidenceComputationTime + "\n");
		writer.flush();
	}
	
	public void close() throws Exception {
		writer.close();
	}
}
