grammar Deduction;
@header {
    package reasoner.io.antlr;
}
/*** program ***/
program : (deductionRule | comment | emptyLine)+;
emptyLine : NEWLINE;
comment : DOUBLESLASH (CHARACTER | SINGLEQUOTE | QUOTE | OPEN | CLOSE | AND | NOT | DASH | GREATER | EQUALS | LESSER | NUMBER | KOMMA | SLASH | BEGIN | END | CONSTANTTOSET)* NEWLINE;

/*** deduction rule ***/
deductionRule : head IMPLIES conjunction NEWLINE;
head : literal;

/*** constraint ***/
constraints : (negatedConstraint | constraint | comment | emptyLine)+;
negatedConstraint : NOT OPEN conjunction CLOSE;
constraint : OPEN conjunction CLOSE | conjunction;

/*** label ***/
labels : (label | comment | emptyLine)+;
label : groundedLiteral confidence NEWLINE;
groundedLiteral : relationName OPEN (constant | date) (KOMMA (constant | date))* CLOSE;
confidence : CONFIDENCE;

/*** logical conjunction ***/
conjunction : (negatedLiteral | literal) (AND (negatedLiteral | literal))*;
negatedLiteral : NOT literal;
literal : relationalLiteral | arithmeticLiteral;
relationalLiteral : relationName OPEN argument (KOMMA argument)* CLOSE;
arithmeticLiteral : argument operator argument;
operator : LESSER | GREATER | EQUALS | LESSEREQUALS | GREATEREQUALS;

/*** arguments ***/
relationName : CHARACTER (CHARACTER | NUMBER)*;
argument : constant | variable | date;
variable : CHARACTER (CHARACTER | NUMBER)*;
constant : QUOTE (CONSTANTTOSET | identifier) QUOTE;
identifier : (CHARACTER | KOMMA | NUMBER | OPEN | CLOSE | DASH | NOT | '[' | ']' | '%' | '$' | SINGLEQUOTE)+;

/*** dates ***/
date : SINGLEQUOTE ((year DASH month DASH day) | BEGIN | END) SINGLEQUOTE;
year : NUMBER NUMBER NUMBER NUMBER;
day : NUMBER NUMBER;
month : NUMBER NUMBER;

/*** Lexer ***/
LESSER : '<';
LESSEREQUALS : '<=';
GREATER : '>';
GREATEREQUALS : '>=';
EQUALS : '=';
KOMMA : ',';
OPEN : '(';
CLOSE : ')';
IMPLIES : '<-';
NOT : '~';
AND : '&';
DASH : '-';
QUOTE : '"';
SINGLEQUOTE : '\'';
BEGIN : 'time_begin';
END : 'time_end';
CONSTANTTOSET : 'constant_to_set';
NUMBER : '0'..'9';
CHARACTER : [a-zA-Z_.öíåéüäç/:\*];
WITHESPACE : (' ' | '\t') {skip();};
NEWLINE : '\n' | '\r';
DOUBLESLASH : '//';
SLASH : '/';
DOT : '.';
CONFIDENCE : ('1' DOT '0') | ('0' DOT NUMBER+);