package reasoner.io.weightlearning;

import reasoner.weightlearning.WeightLearningConfig;
import reasoner.weightlearning.problem.ProblemInstance;
import util.io.FileWriter;

public class WeightLearningStatisticsWriter {
	private FileWriter writer;
	private long groundingTime;
	private String evaluateOn;
	
	public WeightLearningStatisticsWriter(String path) throws Exception {
		writer = new FileWriter(path);
		writer.append("FileName\tstrategy\tmultithreading\tevaluateOn\t#tuplesToLearn\t#labeledTuples\t#formulas\t#clauses\tavgTuplesPerFormula\tRun\tInitialError\tError\tRelativeErrorBound\tAbsoluteErrorBound\t#iterations\tmaxIterations\tGroundingTime\tPreprocessingTime\tRunTime\n");
	}
	
	public void setGroundingTime(long groundingTime) {
		this.groundingTime = groundingTime;
	}
	
	public void setEvaluateOn(String evaluateOn) {
		this.evaluateOn = evaluateOn;
	}
	
	public void appendResults(ProblemInstance problem, int run, double initialError, double error, double iterations, long preprocessingTime, long runTime) throws Exception {
		writer.append(problem.fileName + "\t");
		writer.append(WeightLearningConfig.strategyName + "\t");
		writer.append(WeightLearningConfig.multithreading + "\t");
		if (evaluateOn != null) {
			writer.append(evaluateOn + "\t");
		} else {
			writer.append("null\t");
		}
		writer.append(problem.tuples.length + "\t");
		writer.append(problem.labeledTupleIds.size() + "\t");
		writer.append(problem.formulas.length + "\t");
		if (problem.formulas.length == 0) {
			writer.append("0\t");
		} else {
			if (problem.formulas[0].lineage.subLineage != null) {
				writer.append(problem.formulas[0].lineage.subLineage.length + "\t");
			} else {
				writer.append("0\t");
			}
		}
		if (problem.formulas.length > 0) {
			int numTuples = 0;
			for (int i = 0; i < problem.formulas.length; i++) {
				numTuples += problem.formulas[i].lineage.vars.size();
			}
			double averageTuplesPerFormula = numTuples * 1.0 / problem.formulas.length;
			writer.append(averageTuplesPerFormula + "\t");
		} else {
			writer.append("0\t");
		}
		writer.append(run + "\t");
		writer.append(initialError + "\t");
		writer.append(error + "\t");
		writer.append(WeightLearningConfig.relativeErrorBound + "\t");
		writer.append(WeightLearningConfig.absoluteErrorBound + "\t");
		writer.append(iterations + "\t");
		writer.append(WeightLearningConfig.maxIterations + "\t");
		writer.append(groundingTime + "\t");
		writer.append(preprocessingTime + "\t");
		writer.append(runTime + "\n");
		writer.flush();
	}
	
	public void close() throws Exception {
		writer.close();
	}
}
