package reasoner.lineage;

public class LineageCacheClearer implements LineageVisitor {
	
	private int variableToClear;
	
	public LineageCacheClearer() {
		variableToClear = -1;
	}
	
	public void clearCache(Lineage lineage, int variableToClear) throws Exception {
		this.variableToClear = variableToClear;
		lineage.acceptVisitor(this);
		this.variableToClear = -1;
	}

	public void visit(LineageAnd and) throws Exception {
		and.cachedConf = -1.0;
		for (Lineage child : and.subLineage) {
			if (variableToClear == -1 || child.vars.contains(variableToClear)) {
				child.acceptVisitor(this);
			}
		}
	}

	public void visit(LineageOr or) throws Exception {
		or.cachedConf = -1.0;
		for (Lineage child : or.subLineage) {
			if (variableToClear == -1 || child.vars.contains(variableToClear)) {
				child.acceptVisitor(this);
			}
		}
	}

	public void visit(LineageDisjointOr disjointOr) throws Exception {
		disjointOr.cachedConf = -1.0;
		for (Lineage child : disjointOr.subLineage) {
			if (variableToClear == -1 || child.vars.contains(variableToClear)) {
				child.acceptVisitor(this);
			}
		}
	}

	public void visit(LineageNot not) throws Exception {
		not.cachedConf = -1.0;
		not.subLineage[0].acceptVisitor(this);
	}

	public void visit(LineageLeaf leaf) throws Exception {
		leaf.cachedConf = -1.0;
	}

	public void visit(LineageTrue lineage) throws Exception {
		// Empty
	}

	public void visit(LineageFalse lineage) throws Exception {
		// Empty
	}

}
