package reasoner.lineage;

/**
 * Disjoint Or (as it would result from a Shannon expansion)
 * children: [leaf = true, leaf = false]
 */
public class LineageDisjointOr extends Lineage {
	public LineageLeaf leaf;
	
	public void setVarId(LineageLeaf leaf) {
		this.leaf = leaf;
		varId2count.clear();  // no shannon expansions, since the assignment-space is disjoint
		// we need to override the maxShannon, since it is computed based on the old varId2count
		if (subLineage[0].maxShannon > subLineage[1].maxShannon) {
			maxShannon = subLineage[0].maxShannon;
		} else {
			maxShannon = subLineage[1].maxShannon;
		}
		vars.add(leaf.id);
	}

	@Override
	public void acceptVisitor(LineageVisitor visitor) throws Exception {
		visitor.visit(this);
	}

	@Override
	public String name() {
		return "DISJOINT_OR";
	}
	
	public void clear() {
		leaf = null;
		super.clear();
	}
}
