package reasoner.query;

import java.util.LinkedList;

import reasoner.grounding.Tuple;
import reasoner.io.QueryStatisticsWriter;
import reasoner.query.QuerySettings;
import reasoner.query.QueryExecuter.Results;
import util.io.Paths;

public class RunQuery {
  
	public static void main(String[] args) throws Exception {
		QueryExecuter executer = new QueryExecuter();
		
	  // === Settings ===
	  final int runs = 1;

	  // disable constraints here
	  QuerySettings.useConstraints = true;
	  
	  // disable confidence computations here
	  QuerySettings.computeConfidences = true;
	  
	  boolean printResults = true;
	  // === End Settings ===
	  
		if (args.length != 2) {
			System.err.println("necessary arguments: input_folder output_folder");
			System.err.println("the folder must contain: deductionRules.txt queryRelations.txt dataset.txt");
			System.err.println("the folder can contain: constraints.txt constants.txt");
			return;
		}
		for (int i = 0; i < 2; i++) {
			args[i] = Paths.removeSlash(args[i]);
		}
		QuerySettings.pathToFolder = args[0];
		QueryStatisticsWriter statisticsWriter = new QueryStatisticsWriter(args[1] + "/statistics.tsv");
		executer.initializeSettings();
		while (executer.hasNextQuery()) {
			String constant = executer.getCurrentConstant();
			System.out.println("=== " + constant + " ===");
			for (int run = 0; run < runs; run++) {
			  System.out.println("== run " + run + " ==");
			  Results results = executer.groundQuery();
			  if (QuerySettings.computeConfidences) {
			  	executer.computeConfidences(results);
			  }
			  if (printResults) {
			  	printResultTuples(results.resultTuples);
			  }
			  statisticsWriter.appendResults(constant, run, results);
			  executer.clearQuery(results);
			}
		}
		statisticsWriter.close();
		executer.clean();
	}
	
	private static void printResultTuples(LinkedList<Tuple> results) {
		for (Tuple tuple : results){
      System.out.println(tuple.toString());
    }
	}
}
