package reasoner.query.parsed;

public class ArithmeticLiteral {
	public Operator operator;
	public final AbstractArgument[] arguments;
	public final boolean negated;
	
	public ArithmeticLiteral(boolean negated) {
		arguments = new AbstractArgument[2];
		this.negated = negated;
	}
	
	public void addArgument(AbstractArgument arg) {
		if (arguments[0] == null) {
			arguments[0] = arg;
		} else {
			arguments[1] = arg;
		}
	}
	
	/**
	 * Checks for consistency of arguments
	 */
	public void consistencyCheck() throws Exception {
		if ((arguments[0].isTemporal && !arguments[1].isTemporal)
				|| (!arguments[0].isTemporal && arguments[1].isTemporal)) {
			throw new Exception("Can only compare two arguments of same type: " + this);
		}
		if (operator != Operator.EQUALS) {
			if (!arguments[0].isTemporal || !arguments[1].isTemporal) {
				throw new Exception("Less, greater comparison must have temporal arguments: " + this);
			}
		}
	}
	
	/**
	 * All supported arithemtic operators.
	 * LESS, GREATER must have temporal arguments.
	 * EQUALS can compare any two arguments of the same type.
	 */
	public enum Operator {
		EQUALS ("="),
		LESS ("<"),
		LESSEQUALS ("<="),
		GREATER (">"),
		GREATEREQUALS(">=");
		
		private final String name;
		
		private Operator(String name) {
			this.name = name;
		}
		
		public String toString() {
			return name;
		}
	}
	
	@Override
	public String toString() {
		StringBuffer buffer = new StringBuffer();
		buffer.append(arguments[0]);
		buffer.append(operator);
		return buffer.toString();
	}
}
