package reasoner.query.parsed;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.util.HashMap;

import reasoner.io.DatabaseManager;
import reasoner.io.dataset.Dataset;
import reasoner.query.compiled.CompiledConstraint;

public class ParsedConstraint {
	public final boolean negated;
	public ParsedConjunction conjunction;
	
	public ParsedConstraint(boolean negated) {
		this.negated = negated;
	}
	
	public CompiledConstraint compileConstraint(Dataset dataset) throws Exception {
		StringBuffer from = new StringBuffer();
		StringBuffer where = new StringBuffer();
		HashMap<RelationalLiteral, String> literal2tableName = new HashMap<RelationalLiteral, String>();
		boolean success = conjunction.inferRelations(dataset);
		if (!success) {
			throw new Exception("Could not resolve all relations in constraint: " + this);
		}
		conjunction.compileConjunction(literal2tableName, from, where);
		
		// select clause: arrayIds
		StringBuffer select = new StringBuffer();
		int numPos = 0;
		for (RelationalLiteral l : conjunction.positiveLiterals) {
			if (!l.relation.isUncertain) {
				continue;
			}
			if (numPos > 0) {
				select.append(", ");
			}
			select.append(literal2tableName.get(l));
			select.append(".array_id");
			numPos += 1;
		}
		
		int numNeg = 0;
		for (RelationalLiteral l : conjunction.negativeLiterals) {
			if (!l.relation.isUncertain) {
				continue;
			}
			select.append(", ");
			select.append(literal2tableName.get(l));
			select.append(".array_id");
			numNeg += 1;
		}
		
		// compile SQL stmt
		StringBuffer sql = new StringBuffer();
		sql.append("SELECT ");
		sql.append(select);
		sql.append(" FROM ");
		sql.append(from);
		if (where.length() > 0) {
			sql.append("WHERE ");
			sql.append(where);
		}
		Connection con = DatabaseManager.getInstance().getConnection();
		PreparedStatement stmt = con.prepareStatement(sql.toString());
		
		return new CompiledConstraint(numPos, numNeg, negated, stmt);
	}
}
