package reasoner.query.parsed;

import java.util.HashMap;
import java.util.HashSet;

import util.datastructures.directedgraph.DepthFirstSearch;
import util.datastructures.directedgraph.DirectedEdge;
import util.datastructures.directedgraph.Vertex;

/**
 * This class encapsulates a check for
 * recursive deduction rules. It is based
 * on a directed graph, which is analysed
 * for cycles.
 */
public class RecursionCheck {
	private HashMap<String, Vertex> relation2vertex;
	private Vertex bodyVertex = null;
	private HashSet<String> relationsToCheck;
	
	public RecursionCheck() {
		relation2vertex = new HashMap<String, Vertex>();
		relationsToCheck = new HashSet<String>();
	}
	
	public void addRelationToCheck(String relationName) {
		relationsToCheck.add(relationName);
	}
	
	/**
	 * Performs the recursion check.
	 */
	public void performCheck() throws Exception {
		for (String relationName : relationsToCheck) {
			DepthFirstSearch dfs = new DepthFirstSearch();
			if(dfs.hasCycle(relation2vertex.get(relationName))) {
				throw new Exception("Recursive rules present, starting at relation: " + relationName);
			}
		}
	}
	
	/**
	 * Creates the vertex:
	 * bodyVertex
	 * 
	 * Creates the edge:
	 * headRelation <- bodyVertex
	 */
	public void newRule(String headRelation) throws Exception {
		if (bodyVertex != null) {
			throw new Exception("body vertex != null, when entering a new rule");
		}
		createRelation(headRelation);
		bodyVertex = new Vertex("bodyVertex");
		new DirectedEdge(bodyVertex, relation2vertex.get(headRelation));
	}
	
	/**
	 * Creates the edge:
	 * bodyVertex <- bodyRelation
	 */
	public void addBodyRelation(String bodyRelation) throws Exception {
		if (bodyVertex == null) {
			throw new Exception("no body vertex present, when adding a body relation");
		}
		createRelation(bodyRelation);
		new DirectedEdge(relation2vertex.get(bodyRelation), bodyVertex);
	}
	
	/**
	 * Clears the body vertex.
	 */
	public void exitingRule() throws Exception {
		if (bodyVertex == null) {
			throw new Exception("no body vertex present, when exiting the rule");
		}
		bodyVertex = null;
	}
	
	private void createRelation(String relationName) {
		if (!relation2vertex.containsKey(relationName)) {
			relation2vertex.put(relationName, new Vertex(relationName));
		}
	}
}
