package reasoner.weightlearning;

import java.util.Set;

import reasoner.ReasonerConfig;
import reasoner.io.DatabaseManager;
import reasoner.io.DatasetNameParser;
import reasoner.io.QueryParser;
import reasoner.io.dataset.Dataset;
import reasoner.io.dataset.Relation;
import reasoner.io.weightlearning.ParsedLabels;
import reasoner.io.weightlearning.WeightLearningEpochesWriter;
import reasoner.io.weightlearning.WeightLearningParser;
import reasoner.io.weightlearning.WeightLearningStatisticsWriter;
import reasoner.query.compiled.Query;
import reasoner.weightlearning.problem.ProblemInstance;
import util.io.Paths;

public class RunWeightLearningGrounding {
	
	public static void main(String[] args) throws Exception {
		if (args.length < 2) {
			throw new Exception("Expected arguments: folder_with_query statistics_output_folder\nThe query folder must contain: dataset.txt labels.txt deductionRules.txt relationsToLearn.txt\n");
		}
		for (int i = 0; i < args.length; i++) {
			args[i] = Paths.removeSlash(args[i]);
		}
		
		DatabaseManager manager = DatabaseManager.getInstance();
		manager.clearDBall();
		Dataset dataset = manager.getDataset(DatasetNameParser.getName(args[0] + "/dataset.txt"));

		WeightLearningStatisticsWriter statistics = new WeightLearningStatisticsWriter(args[1] + "/statistics.tsv");
		WeightLearningEpochesWriter epoches = new WeightLearningEpochesWriter(args[1] + "/epoches.tsv");
		WeightLearningExecuter executer = new WeightLearningExecuter(statistics, epoches);
		
		QueryParser queryParser = new QueryParser(dataset);
		WeightLearningGrounder grounder = new WeightLearningGrounder();
		Query query = queryParser.parseQueryPlan(args[0] + "/deductionRules.txt");
		WeightLearningParser labelParser = new WeightLearningParser();

		new WeightLearningConfig(args[1]);
		
		ParsedLabels labels = labelParser.parseLabels(args[0] + "/labels.txt", dataset);
		Set<Relation> relationsToLearn = labelParser.parseRelationsToLearn(args[0] + "/relationsToLearn.txt", dataset);
		for (int run = 0; run < WeightLearningConfig.numRuns; run++) {
			long groundingTime = grounder.createLineage(query);
			statistics.setGroundingTime(groundingTime);
			ProblemInstance problem = grounder.getProblemInstance(args[0], relationsToLearn, labels);
			executer.runCurrentConfig(problem, run);
			problem.clear();
			problem.delete();
			manager.clearDBintensional();
			System.gc();
		}
		ReasonerConfig.clearAll();
		statistics.close();
		epoches.close();
	}
}
