package reasoner.weightlearning;

import reasoner.io.weightlearning.WeightLearningEpochesWriter;
import reasoner.io.weightlearning.WeightLearningStatisticsWriter;
import reasoner.weightlearning.problem.ProblemInstance;
import reasoner.weightlearning.strategies.GradientDescent;
import reasoner.weightlearning.strategies.StochasticGradientDescent;
import reasoner.weightlearning.strategies.WeightLearningOptimizer;
import reasoner.weightlearning.strategies.learningrate.GlobalLearningRate;
import reasoner.weightlearning.strategies.learningrate.LearningRate;
import reasoner.weightlearning.strategies.learningrate.PerTupleLearningRate;

public class WeightLearningExecuter {

	private WeightLearningEpochesWriter epochesWriter;
	private WeightLearningStatisticsWriter statisticsWriter;
	
	public WeightLearningExecuter(WeightLearningStatisticsWriter statisticsWriter, WeightLearningEpochesWriter epochesWriter) throws Exception {
		this.statisticsWriter = statisticsWriter;
		this.epochesWriter = epochesWriter;
	}
	
	public void runCurrentConfig(ProblemInstance problem, int run) throws Exception {
		System.out.println("Running: " + problem.fileName);
		
		WeightLearningOptimizer optimizer = null;
		if (WeightLearningConfig.strategyName.equals("GD")) {
			LearningRate learningRate = new GlobalLearningRate(problem);
			optimizer = new GradientDescent(problem, learningRate, epochesWriter);
		} else if (WeightLearningConfig.strategyName.equals("SGD+PT")) {
			LearningRate learningRate = new PerTupleLearningRate(problem);
			optimizer = new StochasticGradientDescent(problem, epochesWriter, learningRate);
		} else if (WeightLearningConfig.strategyName.equals("SGD")) {
			LearningRate learningRate = new GlobalLearningRate(problem);
			optimizer = new StochasticGradientDescent(problem, epochesWriter, learningRate);
		} else throw new Exception("Unknown optimization strategy: " + WeightLearningConfig.strategyName);
		
		double error = optimizer.run();
		System.out.println(optimizer.getRunTime() + " ms");
		System.out.println("Error: " + error);
		if (statisticsWriter != null) {
			statisticsWriter.appendResults(problem, run, optimizer.getInitialError(), error, optimizer.getNumIterations(), optimizer.getPreprocessingTime(), optimizer.getRunTime());
		}
	}
}
