package reasoner.weightlearning.objective;

import gnu.trove.iterator.TIntIterator;
import reasoner.lineage.confidence.ConfidenceCalculator;
import reasoner.weightlearning.problem.ProblemInstance;
import reasoner.weightlearning.strategies.TupleConfidenceState;

public class L1labelsL1priorObjective extends WeightLearningObjective {
	
	public L1labelsL1priorObjective() throws Exception {
		super();
	}
	
	@Override
	public void init(ProblemInstance problem) throws Exception {
		// empty
	}

	@Override
	public double calculateTupleGradient(int tupleId, ProblemInstance problem, TupleConfidenceState state, ConfidenceCalculator confidenceCalculator) throws Exception {
		double tuplesGradient = 0.0;
		TIntIterator it = problem.tuples[tupleId].labeledFormulas.iterator();
		double exp = Math.exp(state.tupleWeights[tupleId]);
		for (int j = 0; j < problem.tuples[tupleId].labeledFormulas.size(); j++) {
			int formulaId = it.next();
			if (problem.formulas[formulaId].label < state.formulaConfidences[formulaId]) {
				tuplesGradient -= confidenceCalculator.derivative(problem.formulas[formulaId].lineage, problem.tuples[tupleId].leaf.id)
													* ((exp * (1 + exp) - exp * exp) / ((exp + 1) * (exp + 1))) / problem.formulas.length;
			}
			if (problem.formulas[formulaId].label > state.formulaConfidences[formulaId]) {
				tuplesGradient += confidenceCalculator.derivative(problem.formulas[formulaId].lineage, problem.tuples[tupleId].leaf.id)
													* ((exp * (1 + exp) - exp * exp) / ((exp + 1) * (exp + 1))) / problem.formulas.length;
			}
		}
		return tuplesGradient;
	}

	@Override
	public double getFormulaError(double label, double formulaConf) throws Exception {
		return Math.abs(label - formulaConf);
	}

	@Override
	public double getPriorError(double prior, double tupleConf) throws Exception {
		return Math.abs(prior - tupleConf);
	}

}
