package reasoner.weightlearning.objective;

import gnu.trove.TIntCollection;
import gnu.trove.set.hash.TIntHashSet;
import reasoner.lineage.Lineage;
import reasoner.lineage.LineageArray;
import reasoner.lineage.LineageCacheClearer;
import reasoner.lineage.confidence.ConfidenceCalculator;
import reasoner.lineage.confidence.TupleIndependentConfidence;
import reasoner.lineage.providers.LineageProvider;
import reasoner.weightlearning.problem.ProblemInstance;
import reasoner.weightlearning.strategies.TupleConfidenceState;

public class LogicalObjective extends WeightLearningObjective {
	
	private Lineage objectiveFunction;
	private TIntHashSet formulaIds;
	private ConfidenceCalculator confidenceCalculator;
	private LineageCacheClearer cacheClearer;
	
	@Override
	public void init(ProblemInstance problem) throws Exception {
		formulaIds = new TIntHashSet();
		for (int i = 0; i < problem.formulas.length; i++) {
			formulaIds.add(i);
		}
		
		confidenceCalculator = new TupleIndependentConfidence();
		cacheClearer = new LineageCacheClearer();
		
		Lineage[] lineage = new Lineage[problem.formulas.length];
		for (int i = 0; i < problem.formulas.length; i++) {
			if (problem.formulas[i].label == 1.0) {
				lineage[i] = problem.formulas[i].lineage;
			} else if (problem.formulas[i].label == 0.0) {
				lineage[i] = LineageProvider.getInstance().getLineageNot(problem.formulas[i].lineage);
			} else throw new Exception("Cannot handle non-Boolean labels");
		}
		objectiveFunction = LineageProvider.getInstance().getLineageNot(LineageProvider.getInstance().getLineageAnd(lineage));
		LineageArray.getInstance().addLineage(objectiveFunction);
	}
	
	@Override
	public double calculateError(ProblemInstance problem, TupleConfidenceState state, TIntCollection formulaIds, int... tupleIds) throws Exception {
		for (int i = 0; i < tupleIds.length; i++) {
			cacheClearer.clearCache(objectiveFunction, problem.tuples[tupleIds[i]].leaf.id);
		}
		return confidenceCalculator.confidence(objectiveFunction);
	}

	@Override
	public double calculateTupleGradient(int tupleId, ProblemInstance problem,
			TupleConfidenceState state, ConfidenceCalculator confidenceCalculator)
			throws Exception {
		int varId = problem.tuples[tupleId].leaf.id;
		if (!objectiveFunction.vars.contains(varId)) {
			return 0.0;
		}
		double exp = Math.exp(state.tupleWeights[tupleId]);
		return -confidenceCalculator.derivative(objectiveFunction, varId)
    			 * ((exp * (1 + exp) - exp * exp) / ((exp + 1) * (exp + 1)));
	}

	@Override
	protected double getFormulaError(double label, double formulaConf)
			throws Exception {
		throw new Exception("Should not be called");
	}

	@Override
	protected double getPriorError(double prior, double tupleConf)
			throws Exception {
		throw new Exception("No support for priors!");
	}

}
