package reasoner.weightlearning.objective;

import gnu.trove.TIntCollection;
import gnu.trove.iterator.TIntIterator;
import reasoner.lineage.confidence.ConfidenceCalculator;
import reasoner.weightlearning.WeightLearningConfig;
import reasoner.weightlearning.problem.ProblemInstance;
import reasoner.weightlearning.strategies.TupleConfidenceState;

public abstract class WeightLearningObjective {
	
	public abstract void init(ProblemInstance problem) throws Exception;
	
	protected abstract double getFormulaError(double label, double formulaConf) throws Exception;
	
	protected abstract double getPriorError(double prior, double tupleConf) throws Exception;
	
	public abstract double calculateTupleGradient(int tupleId, ProblemInstance problem, TupleConfidenceState state, ConfidenceCalculator confidenceCalculator) throws Exception;
	
	/**
	 * calculates the error for a collection of tuples and a collection of formulas
	 */
	public double calculateError(ProblemInstance problem, TupleConfidenceState state, TIntCollection formulaIds, int... tupleIds) throws Exception {
		double error = 0.0;
		
		TIntIterator it = formulaIds.iterator();
		int numFormulas = formulaIds.size();
		for (int i = 0; i < numFormulas; i++) {
			int formulaId = it.next();
			error += getFormulaError(problem.formulas[formulaId].label, state.formulaConfidences[formulaId]);
		}
		error /= state.formulaConfidences.length;
		return error;
	}
	
	public boolean moreIterationsNecessary(double oldError, double newError, int iteration, int threads) {
		if (iteration < 10) {
			// the learning rate adaption requires a burn in
			return iteration < WeightLearningConfig.maxIterations
					&& newError > WeightLearningConfig.absoluteErrorBound / threads;
		} else {
			double relativeError = (oldError - newError) / oldError;
			return (iteration < WeightLearningConfig.maxIterations &&
					relativeError > WeightLearningConfig.relativeErrorBound / threads
					&& newError > WeightLearningConfig.absoluteErrorBound / threads);
		}
	}
}
