package reasoner.weightlearning.strategies.learningrate;

import reasoner.weightlearning.WeightLearningConfig;
import reasoner.weightlearning.problem.ProblemInstance;
import util.io.FileWriter;

public class GlobalLearningRate implements LearningRate {
	
	private final int numTuples;
	private int numFailed;
	private double learningRate;
	
	public GlobalLearningRate(ProblemInstance problem) throws Exception {
		numTuples = problem.tuples.length;
	}

	@Override
	public void registerResult(int tupleId, boolean improved) throws Exception {
		if (!improved) {
			numFailed++;
		}
	}

	@Override
	public void newEpoche() throws Exception {
		if (numFailed < numTuples * 0.5) {
			learningRate *= 2;
		}
		if (numFailed > numTuples * 0.5) {
			learningRate /= 2;
		}
		numFailed = 0;
	}

	@Override
	public double getLearningRate(int tupleId) throws Exception {
		return learningRate;
	}

	@Override
	public void init() throws Exception {
		numFailed = 0;
		learningRate = WeightLearningConfig.initialLearningRate;
	}

	@Override
	public boolean multithreadable() {
		return false;
	}
	
	@Override
	public void writeToFile(String path) throws Exception {
		FileWriter writer = new FileWriter(path);
		writer.append(learningRate + "\n");
		writer.close();
	}
}
