package reasoner.weightlearning.strategies.learningrate;

import reasoner.weightlearning.WeightLearningConfig;
import reasoner.weightlearning.problem.ProblemInstance;
import util.io.FileWriter;

public class PerTupleLearningRate implements LearningRate {
	
	private final double[] learningRate;
	
	public PerTupleLearningRate(ProblemInstance problem) {
		learningRate = new double[problem.tuples.length];
	}

	@Override
	public void registerResult(int tupleId, boolean improved) throws Exception {
		if (improved) {
			learningRate[tupleId] *= 2;
		} else {
			learningRate[tupleId] /= 2;
		}
	}

	@Override
	public void newEpoche() throws Exception {
		// empty
	}

	@Override
	public double getLearningRate(int tupleId) throws Exception {
		return learningRate[tupleId];
	}

	@Override
	public void init() throws Exception {
		for (int i = 0; i < learningRate.length; i++) {
			learningRate[i] = WeightLearningConfig.initialLearningRate;
		}
	}
	
	@Override
	public boolean multithreadable() {
		return true;
	}
	
	@Override
	public void writeToFile(String path) throws Exception {
		FileWriter writer = new FileWriter(path);
		for (int i = 0; i < learningRate.length; i++) {
			writer.append(i + "\t" + learningRate[i] + "\n");
		}
		writer.close();
	}
}
