package tests.reasoner.grounding;

import static org.junit.Assert.*;

import java.util.LinkedList;

import org.junit.Test;

import reasoner.grounding.Interval;
import reasoner.grounding.IntervalLogic;
import reasoner.grounding.IntervalProvider;

public class IntervalLogicTest extends IntervalLogic {

  public IntervalLogicTest() throws Exception {
    super();
  }

  @Test
  public void makeNonOverlapping_nonOverlapping() throws Exception {
    LinkedList<Interval> inputList = new LinkedList<Interval>();
    addInterval(inputList, 1, 3, 1);
    addInterval(inputList, 4, 8, 2, 3);
    
    LinkedList<Interval> outputList = makeNonOverlapping(inputList);
    LinkedList<Interval> expected= new LinkedList<Interval>();
    addInterval(expected, 1, 3, 1);
    addInterval(expected, 4, 8, 2, 3);
    
    checkIntervals(expected, outputList);
  }

  @Test
  public void makeNonOverlapping_inside() throws Exception {
    LinkedList<Interval> inputList = new LinkedList<Interval>();
    addInterval(inputList, 1, 8, 1);
    addInterval(inputList, 2, 3, 2);
   
    LinkedList<Interval> outputList = makeNonOverlapping(inputList);
    
    LinkedList<Interval> expected = new LinkedList<Interval>();
    addInterval(expected, 1, 2, 1);
    addInterval(expected, 2, 3, 1, 2);
    addInterval(expected, 3, 8, 1);
    
    checkIntervals(expected, outputList);
  }

  @Test
  public void makeNonOverlapping_overlap() throws Exception {
    LinkedList<Interval> inputList = new LinkedList<Interval>();
    addInterval(inputList, 1, 8, 1);
    addInterval(inputList, 4, 10, 2);
   
    LinkedList<Interval> outputList = makeNonOverlapping(inputList);
    
    LinkedList<Interval> expected = new LinkedList<Interval>();
    addInterval(expected, 1, 4, 1);
    addInterval(expected, 4, 8, 1, 2);
    addInterval(expected, 8, 10, 2);
    
    checkIntervals(expected, outputList);
  }

  @Test
  public void makeNonOverlapping_left() throws Exception {
    LinkedList<Interval> inputList = new LinkedList<Interval>();
    addInterval(inputList, 1, 8, 1);
    addInterval(inputList, 1, 10, 2);
   
    LinkedList<Interval> outputList = makeNonOverlapping(inputList);
    
    LinkedList<Interval> expected = new LinkedList<Interval>();
    addInterval(expected, 1, 8, 1, 2);
    addInterval(expected, 8, 10, 2);
    
    checkIntervals(expected, outputList);
  }

  @Test
  public void makeNonOverlapping_right() throws Exception {
    LinkedList<Interval> inputList = new LinkedList<Interval>();
    addInterval(inputList, 4, 8, 1);
    addInterval(inputList, 1, 8, 2);
   
    LinkedList<Interval> outputList = makeNonOverlapping(inputList);
    
    LinkedList<Interval> expected = new LinkedList<Interval>();
    addInterval(expected, 1, 4, 2);
    addInterval(expected, 4, 8, 1, 2);
    
    checkIntervals(expected, outputList);
  }

  @Test
  public void makeNonOverlapping_touching() throws Exception {
    LinkedList<Interval> inputList = new LinkedList<Interval>();
    addInterval(inputList, 4, 8, 1);
    addInterval(inputList, 1, 4, 1);
   
    LinkedList<Interval> outputList = makeNonOverlapping(inputList);
    
    LinkedList<Interval> expected = new LinkedList<Interval>();
    addInterval(expected, 1, 8, 1);
    
    checkIntervals(expected, outputList);
  }

  @Test
  public void makeNonOverlapping_sameArrayIds() throws Exception {
    LinkedList<Interval> inputList = new LinkedList<Interval>();
    addInterval(inputList, 4, 8, 1);
    addInterval(inputList, 2, 6, 1);
   
    LinkedList<Interval> outputList = makeNonOverlapping(inputList);
    
    LinkedList<Interval> expected = new LinkedList<Interval>();
    addInterval(expected, 2, 8, 1);
    
    checkIntervals(expected, outputList);
  }

  @Test
  public void mergeIntervals_nonOverlapping() throws Exception {
    LinkedList<Interval> newList = new LinkedList<Interval>();
    addInterval(newList, 1, 3, 1);
    addInterval(newList, 6, 8, 1);
    
    LinkedList<Interval> oldList = new LinkedList<Interval>();
    addInterval(oldList, 4, 6, 2);
   
    LinkedList<Interval> outputList = mergeIntervals(oldList, newList);
    
    LinkedList<Interval> expected = new LinkedList<Interval>();
    addInterval(expected, 1, 3, 1);
    addInterval(expected, 4, 6, 2);
    addInterval(expected, 6, 8, 1);
    
    checkIntervals(expected, outputList);
  }

  @Test
  public void mergeAdjacentIntervals_nothingToMerge() throws Exception {
    LinkedList<Interval> inputList = new LinkedList<Interval>();
    addInterval(inputList, 1, 4, 1);
    addInterval(inputList, 5, 6, 1);
    addInterval(inputList, 6, 8, 2);
    
    LinkedList<Interval> outputList = mergeAdjacentIntervals(inputList);
    checkIntervals(inputList, outputList);
  }

  @Test
  public void mergeAdjacentIntervals_mergeAndStop() throws Exception {
    LinkedList<Interval> inputList = new LinkedList<Interval>();
    addInterval(inputList, 1, 4, 1);
    addInterval(inputList, 4, 6, 1);
    addInterval(inputList, 6, 8, 2);

    LinkedList<Interval> outputList = mergeAdjacentIntervals(inputList);
    
    LinkedList<Interval> expected = new LinkedList<Interval>();
    addInterval(expected, 1, 6, 1);
    addInterval(expected, 6, 8, 2);
    
    checkIntervals(expected, outputList);
  }

  @Test
  public void mergeAdjacentIntervals_mergeThreeIntervals() throws Exception {
    LinkedList<Interval> inputList = new LinkedList<Interval>();
    addInterval(inputList, 1, 4, 1);
    addInterval(inputList, 4, 6, 1);
    addInterval(inputList, 6, 8, 1);

    LinkedList<Interval> outputList = mergeAdjacentIntervals(inputList);
    
    LinkedList<Interval> expected = new LinkedList<Interval>();
    addInterval(expected, 1, 8, 1);
    
    checkIntervals(expected, outputList);
  }

  @Test
  public void mergeIntervals_overlapping() throws Exception {
    LinkedList<Interval> newList = new LinkedList<Interval>();
    addInterval(newList, 1, 3, 1);
    addInterval(newList, 3, 8, 2);
    
    LinkedList<Interval> oldList = new LinkedList<Interval>();
    addInterval(oldList, 2, 6, 3);
   
    LinkedList<Interval> outputList = mergeIntervals(oldList, newList);
    
    LinkedList<Interval> expected = new LinkedList<Interval>();
    addInterval(expected, 1, 2, 1);
    addInterval(expected, 2, 3, 1, 3);
    addInterval(expected, 3, 6, 2, 3);
    addInterval(expected, 6, 8, 2);
    
    checkIntervals(expected, outputList);
  }

  @Test
  public void mergeIntervals_overlapping2() throws Exception {
    LinkedList<Interval> newList = new LinkedList<Interval>();
    addInterval(newList, 1, 3, 1);
    addInterval(newList, 3, 8, 2);
    
    LinkedList<Interval> oldList = new LinkedList<Interval>();
    addInterval(oldList, 2, 6, 3);
    LinkedList<Interval> outputList = mergeIntervals(newList, oldList);
    
    LinkedList<Interval> expected = new LinkedList<Interval>();
    addInterval(expected, 1, 2, 1);
    addInterval(expected, 2, 3, 1, 3);
    addInterval(expected, 3, 6, 2, 3);
    addInterval(expected, 6, 8, 2);
    checkIntervals(expected, outputList);
  }
  
  @Test
  public void mergeIntervals_overlappingSameId() throws Exception {
    LinkedList<Interval> newList = new LinkedList<Interval>();
    addInterval(newList, 1, 3, 1);
    addInterval(newList, 3, 8, 2);
    
    LinkedList<Interval> oldList = new LinkedList<Interval>();
    addInterval(oldList, 2, 6, 1);
   
    LinkedList<Interval> outputList = mergeIntervals(oldList, newList);
    
    LinkedList<Interval> expected = new LinkedList<Interval>();
    addInterval(expected, 1, 3, 1);
    addInterval(expected, 3, 6, 1, 2);
    addInterval(expected, 6, 8, 2);
    
    checkIntervals(expected, outputList);
  }
  
  @Test
  public void mergeIntervals_overlappingSameLimits() throws Exception {
    LinkedList<Interval> newList = new LinkedList<Interval>();
    addInterval(newList, 1, 3, 1);
    addInterval(newList, 3, 8, 2);
    
    LinkedList<Interval> oldList = new LinkedList<Interval>();
    addInterval(oldList, 3, 8, 3);
   
    LinkedList<Interval> outputList = mergeIntervals(oldList, newList);
    
    LinkedList<Interval> expected = new LinkedList<Interval>();
    addInterval(expected, 1, 3, 1);
    addInterval(expected, 3, 8, 2, 3);
    
    checkIntervals(expected, outputList);
  }

  @Test
  public void mergeIntervals_overlappingSameLimits2() throws Exception {
    LinkedList<Interval> oldList = new LinkedList<Interval>();
    addInterval(oldList, 1, 3, 1);
    addInterval(oldList, 3, 8, 2);
    
    LinkedList<Interval> newList = new LinkedList<Interval>();
    addInterval(newList, 1, 8, 3);
   
    LinkedList<Interval> outputList = mergeIntervals(oldList, newList);
    
    LinkedList<Interval> expected = new LinkedList<Interval>();
    addInterval(expected, 1, 3, 1, 3);
    addInterval(expected, 3, 8, 2, 3);
    
    checkIntervals(expected, outputList);
  }


  private void addInterval(LinkedList<Interval> list, int begin, int end, int... arrayIds) throws Exception {
    Interval newInterval = IntervalProvider.getInstance().get(begin, end, arrayIds);
    list.add(newInterval);
  }
  
  private void checkIntervals(LinkedList<Interval> expected, LinkedList<Interval> actual) {
    assertTrue(actual.size() == expected.size());
    for (int listPos = 0; listPos < actual.size(); listPos++) {
      assertEquals(expected.get(listPos).begin, actual.get(listPos).begin);
      assertEquals(expected.get(listPos).end, actual.get(listPos).end);
      assertTrue(expected.get(listPos).lineageArrayIds.containsAll(actual.get(listPos).lineageArrayIds));
      assertTrue(actual.get(listPos).lineageArrayIds.containsAll(expected.get(listPos).lineageArrayIds));
    }
  }
}
