package tests.reasoner.io;

import static org.junit.Assert.*;

import reasoner.grounding.IntervalProvider;
import reasoner.io.DatabaseManager;
import reasoner.io.dataset.Relation;

import org.junit.Before;
import org.junit.Test;

import reasoner.lineage.Lineage;
import reasoner.lineage.LineageArray;
import reasoner.lineage.LineageLeaf;
import util.Doubles;

public class DatabaseManagerTest {
  private DatabaseManager datasetManager;
  
  public DatabaseManagerTest() throws Exception {
  	datasetManager = DatabaseManager.getInstance();
  }
  
  @Before
  public void setUp() throws Exception {
    try {
    	datasetManager.deleteDataset("newOne");
    } catch (Exception e) {
    	
    }
    datasetManager.clearDBall();
    LineageArray.getInstance().clearAll();
  }
  
  @Test
  public void createAndDelete() {
    String datasetName = "newOne";
    try {
      // create
      int datasetId = datasetManager.createDataset(datasetName);
      Relation rel = datasetManager.newRelation(datasetId, "rel1", 1, true, true, true);
      rel.createTable();
      assertNotNull(rel);
      rel = datasetManager.newRelation(datasetId, "rel2", 2, false, true, true);
      rel.createTable();
      assertNotNull(rel);
      rel = datasetManager.newRelation(datasetId, "rel3", 3, false, true, false);
      rel.createTable();
      assertNotNull(rel);
      datasetManager.finalizeDataset(datasetId, 1);
      
      // delete
      datasetManager.deleteDataset(datasetName);
    } catch (Exception e) {
      System.err.println(e.getMessage());
      assertTrue(false);
    }
  }
  
  @Test
  public void createAndChangeName() {
    int datasetId = -1;
    try {
      datasetId = datasetManager.createDataset("newOne");
    } catch (Exception e) {
      System.err.println(e.getMessage());
      assertTrue(false);
    }
    
    try {
      datasetManager.newRelation(Integer.MAX_VALUE, "rel", 1, false, true, true);
      assertTrue(false);
    } catch (Exception e) {
      
    }
    
    try {
      datasetManager.finalizeDataset(Integer.MIN_VALUE, 1);
      assertTrue(false);
    } catch (Exception e) {
      
    }
    
    // clean up
    try {
      datasetManager.finalizeDataset(datasetId, 1);
      datasetManager.deleteDataset("newOne");
    } catch (Exception e) {
      System.err.println(e.getMessage());
      assertTrue(false);
    }
  }
  
  @Test
  public void createInsertAndLoad() {
    try {
      int datasetId = datasetManager.createDataset("newOne");
      
      // create relation
      Relation relation = datasetManager.newRelation(datasetId, "testrelation", 1, true, true, true);
      assertNotNull(relation);
      relation.createTable();
      
      // insert two facts
      String[] args = new String[1];
      args[0] = "arg1";
      relation.addFactToBatch(args, 0.1, IntervalProvider.getInstance().get(10, 20, 1));
      args[0] = "arg2";
      relation.addFactToBatch(args, 0.2, IntervalProvider.getInstance().get(30, 40, 2));
      relation.executeBatch();
      
      datasetManager.finalizeDataset(datasetId, 3);
      
      // lets check, whether we really keep the new dataset
      datasetManager.clearDBall();
      
      datasetManager.getDataset("newOne");
      
      // grounding emulation
      for (int i = 1; i < 3; i++) {
      	Lineage lineage = LineageArray.getInstance().getLineage(i);
        assertTrue(lineage instanceof LineageLeaf);
      }
      LineageArray.getInstance().finalizeGrounding();
      // check lazy confidence loading
      for (int i = 1; i < 3; i++) {
      	Lineage lineage = LineageArray.getInstance().getLineage(i);
        LineageLeaf leaf = (LineageLeaf)lineage;
        assertTrue(Doubles.equals(0.1 * i, leaf.conf));
      }
      
      datasetManager.deleteDataset("newOne");
    } catch (Exception e) {
      System.err.println(e.getMessage());
      assertTrue(false);
    }
  }
}
