package tests.reasoner.io;

import static org.junit.Assert.*;

import org.junit.After;
import org.junit.Before;
import org.junit.Test;

import reasoner.io.DatabaseManager;
import reasoner.io.QueryParser;
import reasoner.io.dataset.Dataset;

/**
 * This class tests negative instances
 * of deduction rules which should throw
 * an exception.
 */
public class QueryParserTest {
	private QueryParser parser;
	private Dataset dataset;
	private String pathToCode;
	
	public QueryParserTest() {
		pathToCode = System.getProperty("user.dir");
	}
	
	@Before
	public void setUp() throws Exception {
		try {
			DatabaseManager.getInstance().deleteDataset("testDataset");
		} catch (Exception e) {
			// good, the dataset did not exist
		}
		
		// create the test dataset
		DatabaseManager manager = DatabaseManager.getInstance();
		manager.clearDBall();
		int datasetId = manager.createDataset("testDataset");
		manager.newRelation(datasetId, "occ_test", 1, true, true, true).createTable();
		manager.newRelation(datasetId, "occ_test2", 2, true, true, true).createTable();
		manager.newRelation(datasetId, "occ_atemporal", 2, false, true, true).createTable();
		manager.finalizeDataset(datasetId, 1);
		dataset = manager.getDataset("testDataset");
		
		parser = new QueryParser(dataset);
	}
	
	@After
	public void cleanUp() throws Exception {
	  DatabaseManager.getInstance().deleteDataset(dataset.name);
	}
	
	@Test
	public void headVariableUnbound() throws Exception {
		try {
			parser.parseQueryPlan(pathToCode + "/input/testing/io/headVariableUnbound.txt");
			assertTrue(false);  // we should not get here
		} catch (Exception e) {
			// good!
		}
	}
	
	@Test
	public void arithmeticVariableUnbound() throws Exception {
		try {
			parser.parseQueryPlan(pathToCode + "/input/testing/io/arithmeticVariableUnbound.txt");
			assertTrue(false);  // we should not get here
		} catch (Exception e) {
			// good!
		}
	}
	
	@Test
	public void negatedVariableUnbound() throws Exception {
		try {
			parser.parseQueryPlan(pathToCode + "/input/testing/io/negatedVariableUnbound.txt");
			assertTrue(false);  // we should not get here
		} catch (Exception e) {
			// good!
		}
	}
	
	@Test
	public void negatedTemporalVariableUnbound() throws Exception {
		try {
			parser.parseQueryPlan(pathToCode + "/input/testing/io/negatedTemporalVariableUnbound.txt");
			assertTrue(false);  // we should not get here
		} catch (Exception e) {
			// good!
		}
	}
	
	@Test
	public void recursionOneRule() throws Exception {
		try {
			parser.parseQueryPlan(pathToCode + "/input/testing/io/recursionOneRule.txt");
			assertTrue(false);  // we should not get here
		} catch (Exception e) {
			// good!
		}
	}
	
	@Test
	public void recursionTwoRules() throws Exception {
		try {
			parser.parseQueryPlan(pathToCode + "/input/testing/io/recursionTwoRules.txt");
			assertTrue(false);  // we should not get here
		} catch (Exception e) {
			// good!
		}
	}
	
	@Test
	public void wrongNumberOfArgsExtensional() throws Exception {
		try {
			parser.parseQueryPlan(pathToCode + "/input/testing/io/wrongNumberOfArgsExtensional.txt");
			assertTrue(false);  // we should not get here
		} catch (Exception e) {
			// good!
		}
	}
	
	@Test
	public void wrongNumberOfArgsExtensional2() throws Exception {
		try {
			parser.parseQueryPlan(pathToCode + "/input/testing/io/wrongNumberOfArgsExtensional2.txt");
			assertTrue(false);  // we should not get here
		} catch (Exception e) {
			// good!
		}
	}
	
	@Test
	public void wrongNumberOfArgsExtensional3() throws Exception {
		try {
			parser.parseQueryPlan(pathToCode + "/input/testing/io/wrongNumberOfArgsExtensional3.txt");
			assertTrue(false);  // we should not get here
		} catch (Exception e) {
			// good!
		}
	}
	
	@Test
	public void differentNumberOfArgsIntensional() throws Exception {
		try {
			parser.parseQueryPlan(pathToCode + "/input/testing/io/differentNumberOfArgsIntensional.txt");
			assertTrue(false);  // we should not get here
		} catch (Exception e) {
			// good!
		}
	}
	
	@Test
	public void wrongTemporalIntensional() throws Exception {
		try {
			parser.parseQueryPlan(pathToCode + "/input/testing/io/wrongTemporalIntensional.txt");
			assertTrue(false);  // we should not get here
		} catch (Exception e) {
			// good!
		}
	}
	
	@Test
	public void wrongTemporalIntensional2() throws Exception {
		try {
			parser.parseQueryPlan(pathToCode + "/input/testing/io/wrongTemporalIntensional2.txt");
			assertTrue(false);  // we should not get here
		} catch (Exception e) {
			// good!
		}
	}
	
	@Test
	public void extensionalInHead() throws Exception {
		try {
			parser.parseQueryPlan(pathToCode + "/input/testing/io/extensionalInHead.txt");
			assertTrue(false);  // we should not get here
		} catch (Exception e) {
			// good!
		}
	}
}
