package tests.reasoner.io;

import static org.junit.Assert.*;

import org.junit.Test;

import reasoner.io.Time;

public class TimeTest {
	private Time time;
	
	public TimeTest() {
		time = Time.getInstance();
	}
	
	@Test
	public void parseForwards() throws Exception {
		time.setParsingOptions("\\.", "?", true, "time_begin", "time_end");
		int[] result = time.string2int("01.01.0001");
		assertTrue(result[0] == 372);
	}
	
	@Test
	public void parseBackwards() throws Exception {
		time.setParsingOptions("-", "?", false, "time_begin", "time_end");
		int[] result = time.string2int("0001-01-01");
		assertTrue(result[0] == 372);
	}
	
	@Test
	public void parseUnknownDay() throws Exception {
		time.setParsingOptions("\\.", "?", true, "time_begin", "time_end");
		int[] result = time.string2int("??.02.1999");
		assertTrue(result[1] - result[0] == 31);
	}
	
	@Test
	public void parseUnknownMonth() throws Exception {
		time.setParsingOptions("\\.", "#", true, "time_begin", "time_end");
		int[] result = time.string2int("##.##.1999");
		assertTrue(result[1] - result[0] == 372);
	}
	
	@Test
	public void parseInterval() throws Exception {
		time.setParsingOptions("\\.", "#", true, "time_begin", "time_end");
		int[] result = time.string2int("##.##.0000--31.12.0001");
		assertTrue(result[1] - result[0] == 372 * 2);
	}
	
	@Test
	public void parseBeginEnd() throws Exception {
	  time.setParsingOptions("\\.", "#", true, "time_begin", "time_end");
	  int[] result = time.string2int("time_begin--time_end");
	  assertTrue(result[0] == time.minTime);
	  assertTrue(result[1] == time.maxTime);
	}
	
	@Test
	public void parseYearPlaceholder() throws Exception {
	  time.setParsingOptions("\\.", "?", true, "time_begin", "time_end");
	  int[] result = time.string2int("??.??.000?");
	  assertTrue(result[1] - result[0] == 372 * 10);
	}
	
	@Test
	public void parseYearPlaceholderTwo() throws Exception {
	  time.setParsingOptions("\\.", "#", true, "time_begin", "time_end");
	  int[] result = time.string2int("##.##.00##");
	  assertTrue(result[1] - result[0] == 372 * 100);
	}
	
	@Test
	public void parseYearPlaceholderIgnoreDayMonth() throws Exception {
	  time.setParsingOptions("\\.", "#", true, "time_begin", "time_end");
	  int[] result = time.string2int("02.02.000#");
	  assertTrue(result[1] - result[0] == 372 * 10);
	}
	
	@Test
	public void negativeYear() throws Exception {
	  time.setParsingOptions("\\.", "#", true, "time_begin", "time_end");
	  int[] result = time.string2int("##.##.-0001");
	  assertTrue(result[1] - result[0] == 372 * 1);
	}
	
	@Test
	public void negativeYearPlaceholder() throws Exception {
	  time.setParsingOptions("\\.", "#", true, "time_begin", "time_end");
	  int[] result = time.string2int("##.##.-001#");
	  assertTrue(result[1] - result[0] == 372 * 10);
	}
}
