package tests.reasoner.lineage.providers;

import static org.junit.Assert.assertTrue;

import org.junit.Before;
import org.junit.Test;

import reasoner.ReasonerConfig;
import reasoner.lineage.Lineage;
import reasoner.lineage.LineageAnd;
import reasoner.lineage.LineageLeaf;
import reasoner.lineage.providers.LineageProvider;

public class LineageProviderTest {
  private LineageProvider provider;
  
  @Before
  public void setUp() throws Exception {
    provider = LineageProvider.getInstance();
  }
  
  @Test
  public void overlappingLineageDeletion() throws Exception {
    Lineage[] allLineage = new Lineage[7];
    allLineage[0] = provider.getLineageLeaf(1, 0.1);
    allLineage[1] = provider.getLineageLeaf(2, 0.2);
    allLineage[2] = provider.getLineageLeaf(3, 0.2);
    allLineage[3] = provider.getLineageNot(allLineage[0]);
    allLineage[4] = provider.getLineageAnd(allLineage[3], allLineage[0]);
    allLineage[5] = provider.getLineageOr(allLineage[4], allLineage[3]);
    allLineage[6] = provider.getLineageOr(allLineage[1], allLineage[2]);
    for (Lineage l : allLineage) {
      provider.handBack(l, true);
    }
    assertTrue(provider.allObjectsReturned());
  }
  
  @Test
  public void testSplitting() throws Exception {
  	assertTrue(ReasonerConfig.useLineagePreprocessing);
  	assertTrue(ReasonerConfig.maxShannonPerNode > 1);
  	LineageLeaf[] leafs = new LineageLeaf[6];
  	for (int i = 0; i < 6; i++) {
  		leafs[i] = provider.getLineageLeaf(i, 0.1 * i);
  	}
  	Lineage[] ors = new Lineage[4];
  	// expansion 1
  	ors[0] = provider.getLineageOr(leafs[0], leafs[1]);
  	ors[1] = provider.getLineageOr(leafs[0], leafs[2]);
  	// expansion 2
  	ors[2] = provider.getLineageOr(leafs[3], leafs[4]);
  	ors[3] = provider.getLineageOr(leafs[3], leafs[5]);
  	// input: (0 \/ 1) /\ (0 \/ 2) /\ (3 \/ 4) /\ (3 \/ 5)
  	Lineage splitted = provider.getLineageAnd(ors);
  	assertTrue(splitted.subLineage.length == 2);
  	for (int i = 0; i < 2; i++) {
  		assertTrue(splitted.subLineage[i] instanceof LineageAnd);
  		assertTrue(splitted.subLineage[i].varId2count.size() == 1);
  		assertTrue(splitted.subLineage[i].subLineage.length == 2);
  	}
  	provider.handBack(splitted, true);
  	assertTrue(provider.allObjectsReturned());
  }
}
