package tests.reasoner.weightlearning.problem;

import static org.junit.Assert.assertTrue;

import java.util.List;

import gnu.trove.map.hash.TIntIntHashMap;

import org.junit.Test;

import reasoner.lineage.Lineage;
import reasoner.lineage.LineageArray;
import reasoner.lineage.LineageLeaf;
import reasoner.lineage.providers.LineageProvider;
import reasoner.weightlearning.problem.ComputationTask;
import reasoner.weightlearning.problem.LabeledFormula;
import reasoner.weightlearning.problem.ProblemInstance;
import reasoner.weightlearning.problem.TupleToLearn;

public class ProblemInstanceTest {

	@Test
	public void inpependentTasksCreation() throws Exception {
		LineageProvider provider = LineageProvider.getInstance();
		LineageArray lineages = LineageArray.getInstance();
		lineages.clearAll();
		
		lineages.addLineage(provider.getLineageLeaf(0, 0.1));
		lineages.addLineage(provider.getLineageLeaf(1, 0.2));
		lineages.addLineage(provider.getLineageLeaf(2, 0.3));
		lineages.addLineage(provider.getLineageLeaf(3, 0.4));
		Lineage and = provider.getLineageAnd(lineages.getLineage(1), lineages.getLineage(2));
		assertTrue(lineages.addLineage(and) == 4);
		
		LabeledFormula[] formulas = new LabeledFormula[3];
		formulas[0] = new LabeledFormula();
		formulas[0].lineage = lineages.getLineage(1);  // leaf with id 1
		formulas[0].label = 0.5;
		formulas[1] = new LabeledFormula();
		formulas[1].lineage = lineages.getLineage(4);  // lineage and
		formulas[1].label = 0.5;
		formulas[2] = new LabeledFormula();
		formulas[2].lineage = lineages.getLineage(3);  // leaf with id 3
		formulas[2].label = 0.5;
		
		TupleToLearn[] tuples = new TupleToLearn[3];
		tuples[0] = new TupleToLearn("1", (LineageLeaf)lineages.getLineage(1));
		tuples[1] = new TupleToLearn("3", (LineageLeaf)lineages.getLineage(3));
		tuples[2] = new TupleToLearn("2", (LineageLeaf)lineages.getLineage(2));
		
		ProblemInstance problem = new ProblemInstance("testInstance", formulas, tuples);
		TIntIntHashMap arrayId2tupleId = problem.collectLabeledTuples(tuples, formulas);
		assertTrue(arrayId2tupleId.size() == 3);
		assertTrue(!arrayId2tupleId.containsKey(0));
		assertTrue(arrayId2tupleId.get(1) == 0);
		assertTrue(arrayId2tupleId.get(2) == 2);
		assertTrue(arrayId2tupleId.get(3) == 1);
		
		List<ComputationTask> tasks = problem.computeIndependentTasks(tuples, formulas, arrayId2tupleId);
		
		assertTrue(tasks.size() == 2);
		ComputationTask task = tasks.get(0);
		assertTrue(task.tupleIds.size() == 1);
		assertTrue(task.tupleIds.contains(1));
		task = tasks.get(1);
		assertTrue(task.tupleIds.size() == 2);
		assertTrue(task.tupleIds.contains(0));
		assertTrue(task.tupleIds.contains(2));
		
		lineages.clearAll();
	}
}
