package tests.reasoner.weightlearning.strategies;

import static org.junit.Assert.assertTrue;

import org.junit.Test;

import reasoner.io.weightlearning.WeightLearningEpochesWriter;
import reasoner.lineage.LineageArray;
import reasoner.lineage.providers.LineageProvider;
import reasoner.weightlearning.WeightLearningConfig;
import reasoner.weightlearning.problem.LabeledFormula;
import reasoner.weightlearning.problem.ProblemInstance;
import reasoner.weightlearning.problem.TupleToLearn;
import reasoner.weightlearning.strategies.StochasticGradientDescent;
import reasoner.weightlearning.strategies.learningrate.PerTupleLearningRate;

public class StochasticGradientDescentTest {

	@Test
	public void multiThreadedTest() throws Exception {
		assertTrue(WeightLearningConfig.multithreading);
		
		// lineage
		LineageArray lineages = LineageArray.getInstance();
		lineages.clearAll();
		LineageProvider provider = LineageProvider.getInstance();
		
		// problem instance
		int numTuples = 400;
		TupleToLearn[] tuples = new TupleToLearn[numTuples];
		LabeledFormula[] formulas = new LabeledFormula[numTuples];
		for (int i = 0; i < numTuples; i++) {
			tuples[i] = new TupleToLearn(i + "", provider.getLineageLeaf(i, 0.5));
			formulas[i] = new LabeledFormula();
			formulas[i].label = 0.5;
			formulas[i].lineage = tuples[i].leaf;
			lineages.addLineage(tuples[i].leaf);
		}
		ProblemInstance problem = new ProblemInstance("testInstance", formulas, tuples);
		assertTrue(problem.independentTasks.size() == 400);
		
		// stochastic gradient descent
		new WeightLearningConfig(null);
		WeightLearningEpochesWriter writer = new WeightLearningEpochesWriter("/tmp/test");
		PerTupleLearningRate learningRate = new PerTupleLearningRate(problem);
		StochasticGradientDescent sgd = new StochasticGradientDescent(problem, writer, learningRate);
		double error = sgd.run();
		assertTrue(error < 0.01);
		
		lineages.clearAll();
	}
}
