package tests.util;

import static org.junit.Assert.assertTrue;

import gnu.trove.list.linked.TDoubleLinkedList;
import gnu.trove.list.linked.TIntLinkedList;
import gnu.trove.map.hash.TIntIntHashMap;

import java.util.HashSet;
import java.util.LinkedList;
import java.util.List;

import org.junit.Test;

import util.Collections;
import util.Doubles;

public class CollectionsTest {

	@Test
	public void testPermutations() {
		Collections<Integer> permutations = new Collections<Integer>();
		LinkedList<Integer> input = new LinkedList<Integer>();
		for (int i = 0; i < 4; i++) {
			input.add(i);
		}
		
		List<List<Integer>> result = permutations.createAllPermutations(input);
		assertTrue(result.size() == 24);  // 24 = 4 * 3 * 2 * 1
		HashSet<List<Integer>> set = new HashSet<List<Integer>>();  // tests whether the permutations are unique
		for (List<Integer> permutation : result) {
			assertTrue(permutation.size() == 4);
			for (int i = 0; i < 4; i++) {
				assertTrue(permutation.contains(i));
			}
			set.add(permutation);
		}
		assertTrue(set.size() == 24);
	}
	
	@Test
	public void normEmpty() {
		TDoubleLinkedList list = new TDoubleLinkedList();
		double norm = Collections.euclideanNorm(list);
		assertTrue(Doubles.equals(norm, 0.0));
	}
	
	@Test
	public void normOneElement() {
		TDoubleLinkedList list = new TDoubleLinkedList();
		list.add(1.0);
		double norm = Collections.euclideanNorm(list);
		assertTrue(Doubles.equals(norm, 1.0));
	}
	
	@Test
	public void normFourElements() {
		TDoubleLinkedList list = new TDoubleLinkedList();
		for (int i = 0; i < 4; i++) {
			list.add(2.0);
		}
		double norm = Collections.euclideanNorm(list);
		assertTrue(Doubles.equals(norm, 4.0));
	}
	
	@Test
	public void replaceInCollectionEmpty() {
		TIntLinkedList list = new TIntLinkedList();
		TIntIntHashMap replacements = new TIntIntHashMap();
		Collections.replaceInCollection(list, replacements);
		assertTrue(list.isEmpty());
		assertTrue(replacements.isEmpty());
	}
	
	@Test
	public void replaceInCollection() {
		TIntLinkedList list = new TIntLinkedList();
		list.add(1);  // should become 3
		list.add(1);  // should become 3
		list.add(2);  // is not replaced
		TIntIntHashMap replacements = new TIntIntHashMap();
		replacements.put(1, 3);
		Collections.replaceInCollection(list, replacements);
		assertTrue(list.size() == 3);
		assertTrue(list.get(0) == 3);
		assertTrue(list.get(1) == 3);
		assertTrue(list.get(2) == 2);
	}
}
