package util;

import gnu.trove.TDoubleCollection;
import gnu.trove.TIntCollection;
import gnu.trove.iterator.TDoubleIterator;
import gnu.trove.iterator.TIntIterator;
import gnu.trove.list.linked.TIntLinkedList;
import gnu.trove.map.TIntIntMap;

import java.util.Collection;
import java.util.LinkedList;
import java.util.List;

public class Collections<T> {
	/**
	 * Creates all permutations of a given list.
	 * Expenonetial run-time in the size of the list. Careful!
	 * 
	 * @param toPermute : list to be permuted
	 * @return ListOfPermutations<Permutation<T>>
	 */
	public List<List<T>> createAllPermutations(Collection<T> toPermute) {
		LinkedList<List<T>> output = new LinkedList<List<T>>();
		createPermutationsRecursive(new LinkedList<T>(), toPermute, output);
		return output;
	}
	
	/**
	 * Recursive enumeration of all permutations.
	 * 
	 * @param myOutput : current permutation state
	 * @param input : list to process and create all permutations
	 * @param allOutput : collects all permutations at the end of the recursion
	 */
	private void createPermutationsRecursive(List<T> myOutput, Collection<T> input, List<List<T>> allOutput) {
		if (input.isEmpty()) {
			// recursion ends
			allOutput.add(myOutput);
		} else {
			// recursive subcall for each element in @input
			for (T element : input) {
				LinkedList<T> inputSubCall = new LinkedList<T>(input);
				inputSubCall.remove(element);
				LinkedList<T> outputSubCall = new LinkedList<T>(myOutput);
				outputSubCall.add(element);
				createPermutationsRecursive(outputSubCall, inputSubCall, allOutput);
			}
		}
	}
	
	public static double euclideanNorm(TDoubleCollection list) {
		double sum = 0.0;
		TDoubleIterator it = list.iterator();
		for (int i = 0; i < list.size(); i++) {
			double val = it.next();
			sum += val * val;
		}
		return Math.sqrt(sum);
	}
	
	/**
	 * Replaces all integer values in a collection.
	 * If @replacements is empty, the collection is not altered.
	 * 
	 * @param collection : the input and output collection with replaced values
	 * @param replacements
	 */
	public static void replaceInCollection(TIntCollection collection, TIntIntMap replacements) {
		TIntLinkedList result = new TIntLinkedList();
		TIntIterator it = collection.iterator();
		int size = collection.size();
		for (int i = 0; i < size; i++) {
			int val = it.next();
			if (replacements.containsKey(val)) {
				int newVal = replacements.get(val);
				result.add(newVal);
			} else {
				result.add(val);
			}
		}
		collection.clear();
		collection.addAll(result);
	}
}
