package util;

public class Doubles {
  /**
   * checks whether two doubles @d1 and @d2 equal each other using @epsilon.
   */
  public static boolean equals(double d1, double d2) {
    if (Math.abs(d1 - d2) > 0.0000000000001) {
    	return false;
    }
    
    long bits1 = Double.doubleToLongBits(d1);
    long bits2 = Double.doubleToLongBits(d2);
    long mantissa1 = (bits1 & 0x000fffffffffffffL) | 0x0010000000000000L;
    long mantissa2 = (bits2 & 0x000fffffffffffffL) | 0x0010000000000000L;
    return Math.abs(mantissa1 - mantissa2) < 1000;
  }
  
  /**
   * Expects inputs in [0.0, 1.0].
   * Will bucketize the interval to [0, ... , @buckets] .
   */
  public static int bucketize(double d, int buckets) throws Exception {
  	if (d < 0.0 || d > 1.0) {
  		throw new Exception("Double outside of [0.0, 1.0]: " + d);
	 	}
  	if (buckets < 1) {
  		throw new Exception("There must be at least one bucket");
  	}
  	return (int)(d * buckets);
  }
}
