package util;

import java.util.Map;
import java.util.Map.Entry;

/**
 * contains various utils for strings
 */
public class Strings {
  
  /**
   * merges all the string starting at @from till @to by appending them.
   * 
   * Example:
   * input: array = {"a", "b", "c", "d"}, from = 1, to = 3
   * results in: {"a", "bc", "d"}
   * 
   * @param array : an array of strings
   * @param from : the first index to be merged
   * @param to : all indices before that will be merged
   * @return
   */
  public static String[] merge(String[] array, int from, int to) throws ArrayIndexOutOfBoundsException {
    if (from < 0 || from >= to || to > array.length) {
      throw new ArrayIndexOutOfBoundsException();
    }
    String[] result = new String[array.length - (to - from) + 1];
    // before
    for (int i = 0; i < from; i++) {
      result[i] = array[i];
    }
    // merge from -> to
    if (from < to) {
      StringBuffer buffer = new StringBuffer();
      for (int i = from; i < to; i++) {
        buffer.append(array[i]);
      }
      result[from] = buffer.toString();
    }
    // after
    for (int i = to; i < array.length; i++) {
      result[from + to + 1 - i] = array[i];
    }
    return result;
  }
  
  /**
   * @param target : the string to replace in
   * @param replacements : <regex, replacement>
   */
  public static String multiReplace(String target, Map<String, String> replacements) {
  	if (replacements.isEmpty()) {
  		return target;
  	}
  	for (Entry<String, String> replacement : replacements.entrySet()) {
  		target = target.replaceAll(replacement.getKey(), replacement.getValue());
  	}
  	return target;
  }
}
