package util.datastructures;

import gnu.trove.map.hash.TIntIntHashMap;

/**
 * Wrapps a map around @UnionFind to allow for all ranges of ids
 */
public class MappedUnionFind extends UnionFind {
  
  private TIntIntHashMap forwards;
  private int[] backwards;
  
  public MappedUnionFind(int... ids) {
    super(ids.length);
    forwards = new TIntIntHashMap();
    backwards = new int[ids.length];
    int i = 0;
    for (int id : ids) {
      forwards.put(id, forwards.size());
      backwards[i++] = id;
    }
  }
  
  @Override
  public int find(int k) {
    return backwards[findImplementation(forwards.get(k))];
  }
  
  @Override
  public void union(int a, int b) {
  	super.union(forwards.get(a), forwards.get(b));
  }
}
