package util.datastructures;

public class UnionFind {
	private int[] rank; 
	private int[] parent; 
	private int numSets;

	public UnionFind(int k){
		this.rank = new int[k];
		this.parent = new int[k];
		for (int i=0; i<k; i++){
			this.parent[i] = i;
			this.rank[i] = 1;
		}
		this.numSets = k;
	}
	
	public int find(int k){
		return findImplementation(k);
	}
	
	/**
	 * needed for union(). Otherwise it calls the more specific find
	 * of subclasses.
	 */
	protected int findImplementation(int k) {
		if (this.parent[k] != k){
			this.parent[k] = this.findImplementation(this.parent[k]);
		}
		return this.parent[k];
	}
	
	public void union(int x, int y){
		int xRoot = this.findImplementation(x);
		int yRoot = this.findImplementation(y);
		
		if (xRoot==yRoot)
			return;
		
		// merge!
		if (this.rank[xRoot] < this.rank[yRoot]){
			this.parent[xRoot] = yRoot;
		}else if (this.rank[xRoot] > this.rank[yRoot]){
			this.parent[yRoot] = xRoot;
		}else{
			this.parent[yRoot] = xRoot;
			this.rank[xRoot]++;
		}
		this.numSets--;
	}
	
	public int numSets(){
		return this.numSets;
	}
}
